@@ -1,18 +1,20 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from __future__ import division
 
 from dateutil.relativedelta import relativedelta
-from trytond.model import ModelView, ModelSQL, Workflow, fields
-from trytond.wizard import Wizard, StateView, StateAction, Button
-from trytond.tools import datetime_strftime
-from trytond.pyson import Eval, If, PYSONEncoder
-from trytond.transaction import Transaction
-from trytond.pool import Pool
 
-__all__ = ['FiscalYear', 'FiscalYearLine',
-    'BalanceNonDeferralStart', 'BalanceNonDeferral',
-    'RenewFiscalYearStart', 'RenewFiscalYear']
+from trytond.i18n import gettext
+from trytond.model import ModelView, ModelSQL, Workflow, fields
+from trytond.model.exceptions import AccessError
+from trytond.pool import Pool
+from trytond.pyson import Eval, If, PYSONEncoder, Id
+from trytond.rpc import RPC
+from trytond.transaction import Transaction
+from trytond.wizard import (
+    Wizard, StateView, StateTransition, StateAction, Button)
+
+from .exceptions import (FiscalYearNotFoundError, FiscalYearDatesError,
+    FiscalYearSequenceError, FiscalYearCloseError, FiscalYearReOpenError)
 
 STATES = {
     'readonly': Eval('state') != 'open',
@@ -31,75 +33,59 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
         domain=[('end_date', '>=', Eval('start_date', None))],
         depends=DEPENDS + ['start_date'])
     periods = fields.One2Many('account.period', 'fiscalyear', 'Periods',
-            states=STATES, depends=DEPENDS)
+        states=STATES,
+        domain=[
+            ('company', '=', Eval('company')),
+            ],
+        depends=DEPENDS + ['company'])
     state = fields.Selection([
             ('open', 'Open'),
             ('close', 'Close'),
             ('locked', 'Locked'),
             ], 'State', readonly=True, required=True)
-    post_move_sequence = fields.Many2One('ir.sequence', 'Post Move Sequence',
-            required=True, domain=[('code', '=', 'account.move'),
-                ['OR',
-                    ('company', '=', Eval('company')),
-                    ('company', '=', None)
-                ]],
-            context={
-                'code': 'account.move',
-                'company': Eval('company'),
-            },
-            depends=['company'])
-    company = fields.Many2One('company.company', 'Company', required=True,
+    post_move_sequence = fields.Many2One(
+        'ir.sequence', "Post Move Sequence", required=True,
         domain=[
-            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', -1)),
-            ], select=True)
-    close_lines = fields.Many2Many('account.fiscalyear-account.move.line',
-            'fiscalyear', 'line', 'Close Lines')
+            ('sequence_type', '=',
+                Id('account', 'sequence_type_account_move')),
+            ('company', '=', Eval('company')),
+            ],
+        depends=['company'])
+    company = fields.Many2One(
+        'company.company', "Company", required=True, select=True)
     icon = fields.Function(fields.Char("Icon"), 'get_icon')
 
     @classmethod
     def __setup__(cls):
         super(FiscalYear, cls).__setup__()
         cls._order.insert(0, ('start_date', 'ASC'))
-        cls._error_messages.update({
-                'change_post_move_sequence': ('You can not change the post '
-                    'move sequence in fiscal year "%s".'),
-                'no_fiscalyear_date': 'No fiscal year defined for "%s".',
-                'fiscalyear_overlaps': ('Fiscal year "%(first)s" and '
-                    '"%(second)s" overlap.'),
-                'different_post_move_sequence': ('Fiscal year "%(first)s" and '
-                    '"%(second)s" have the same post move sequence.'),
-                'account_balance_not_zero': ('The balance of the account "%s" '
-                    'must be zero.'),
-                'close_error': ('You can not close fiscal year "%s" until you '
-                    'close all previous fiscal years.'),
-                'reopen_error': ('You can not reopen fiscal year "%s" until '
-                    'you reopen all later fiscal years.'),
-                })
         cls._transitions |= set((
                 ('open', 'close'),
                 ('close', 'locked'),
                 ('close', 'open'),
                 ))
         cls._buttons.update({
-                'create_period': {
-                    'invisible': ((Eval('state') != 'open')
-                        | Eval('periods', [0])),
-                    },
-                'create_period_3': {
+                'create_periods': {
                     'invisible': ((Eval('state') != 'open')
                         | Eval('periods', [0])),
+                    'depends': ['state'],
                     },
                 'close': {
                     'invisible': Eval('state') != 'open',
+                    'depends': ['state'],
                     },
                 'reopen': {
                     'invisible': Eval('state') != 'close',
+                    'depends': ['state'],
                     },
                 'lock': {
                     'invisible': Eval('state') != 'close',
+                    'depends': ['state'],
                     },
                 })
+        cls.__rpc__.update({
+                'create_period': RPC(readonly=False, instantiate=0),
+                })
 
     @staticmethod
     def default_state():
@@ -111,9 +97,9 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
 
     def get_icon(self, name):
         return {
-            'open': 'tryton-open',
-            'close': 'tryton-close',
-            'locked': 'tryton-readonly',
+            'open': 'tryton-account-open',
+            'close': 'tryton-account-close',
+            'locked': 'tryton-account-block',
             }.get(self.state)
 
     @classmethod
@@ -141,10 +127,10 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
         second_id = cursor.fetchone()
         if second_id:
             second = self.__class__(second_id[0])
-            self.raise_user_error('fiscalyear_overlaps', {
-                    'first': self.rec_name,
-                    'second': second.rec_name,
-                    })
+            raise FiscalYearDatesError(
+                gettext('account.msg_fiscalyear_overlap',
+                    first=self.rec_name,
+                    second=second.rec_name))
 
     def check_post_move_sequence(self):
         years = self.search([
@@ -152,22 +138,30 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
                 ('id', '!=', self.id),
                 ])
         if years:
-            self.raise_user_error('different_post_move_sequence', {
-                    'first': self.rec_name,
-                    'second': years[0].rec_name,
-                    })
+            raise FiscalYearSequenceError(
+                gettext('account.msg_fiscalyear_different_post_move_sequence',
+                    first=self.rec_name,
+                    second=years[0].rec_name))
 
     @classmethod
     def write(cls, *args):
+        pool = Pool()
+        Move = pool.get('account.move')
         actions = iter(args)
         for fiscalyears, values in zip(actions, actions):
             if values.get('post_move_sequence'):
                 for fiscalyear in fiscalyears:
                     if (fiscalyear.post_move_sequence
-                            and fiscalyear.post_move_sequence.id !=
-                            values['post_move_sequence']):
-                        cls.raise_user_error('change_post_move_sequence', (
-                                fiscalyear.rec_name,))
+                            and fiscalyear.post_move_sequence.id
+                            != values['post_move_sequence']):
+                        if Move.search([
+                                    ('period.fiscalyear', '=', fiscalyear.id),
+                                    ('state', '=', 'posted'),
+                                    ]):
+                            raise AccessError(
+                                gettext('account.'
+                                    'msg_change_fiscalyear_post_move_sequence',
+                                    fiscalyear=fiscalyear.rec_name))
         super(FiscalYear, cls).write(*args)
 
     @classmethod
@@ -177,8 +171,7 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
         super(FiscalYear, cls).delete(fiscalyears)
 
     @classmethod
-    @ModelView.button
-    def create_period(cls, fiscalyears, interval=1):
+    def create_period(cls, fiscalyears, interval=1, end_day=31):
         '''
         Create periods for the fiscal years with month interval
         '''
@@ -187,14 +180,15 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
         for fiscalyear in fiscalyears:
             period_start_date = fiscalyear.start_date
             while period_start_date < fiscalyear.end_date:
-                period_end_date = period_start_date + \
-                    relativedelta(months=interval - 1) + \
-                    relativedelta(day=31)
+                month_offset = 1 if period_start_date.day < end_day else 0
+                period_end_date = (period_start_date
+                    + relativedelta(months=interval - month_offset)
+                    + relativedelta(day=end_day))
                 if period_end_date > fiscalyear.end_date:
                     period_end_date = fiscalyear.end_date
-                name = datetime_strftime(period_start_date, '%Y-%m')
-                if name != datetime_strftime(period_end_date, '%Y-%m'):
-                    name += ' - ' + datetime_strftime(period_end_date, '%Y-%m')
+                name = period_start_date.strftime('%Y-%m')
+                if name != period_end_date.strftime('%Y-%m'):
+                    name += ' - ' + period_end_date.strftime('%Y-%m')
                 to_create.append({
                     'name': name,
                     'start_date': period_start_date,
@@ -207,12 +201,9 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
             Period.create(to_create)
 
     @classmethod
-    @ModelView.button
-    def create_period_3(cls, fiscalyears):
-        '''
-        Create periods for the fiscal years with 3 months interval
-        '''
-        cls.create_period(fiscalyears, interval=3)
+    @ModelView.button_action('account.act_create_periods')
+    def create_periods(cls, fiscalyears):
+        pass
 
     @classmethod
     def find(cls, company_id, date=None, exception=True):
@@ -235,13 +226,10 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
             ], order=[('start_date', 'DESC')], limit=1)
         if not fiscalyears:
             if exception:
-                language = Transaction().language
-                languages = Lang.search([('code', '=', language)])
-                if not languages:
-                    languages = Lang.search([('code', '=', 'en')])
-                language, = languages
-                formatted = Lang.strftime(date, language.code, language.date)
-                cls.raise_user_error('no_fiscalyear_date', (formatted,))
+                lang = Lang.get()
+                raise FiscalYearNotFoundError(
+                    gettext('account.msg_no_fiscalyear_date',
+                        date=lang.strftime(date)))
             else:
                 return None
         return fiscalyears[0].id
@@ -252,12 +240,14 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
         Currency = pool.get('currency.currency')
         Deferral = pool.get('account.account.deferral')
 
-        if account.kind == 'view':
+        if not account.type:
             return
         if not account.deferral:
             if not Currency.is_zero(self.company.currency, account.balance):
-                self.raise_user_error('account_balance_not_zero',
-                        error_args=(account.rec_name,))
+                raise FiscalYearCloseError(
+                    gettext('account'
+                        '.msg_close_fiscalyear_account_balance_not_zero',
+                        account=account.rec_name))
         else:
             deferral = Deferral()
             deferral.account = account
@@ -292,7 +282,9 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
                         ('state', '=', 'open'),
                         ('company', '=', fiscalyear.company.id),
                         ]):
-                cls.raise_user_error('close_error', (fiscalyear.rec_name,))
+                raise FiscalYearCloseError(
+                    gettext('account.msg_close_fiscalyear_earlier',
+                        fiscalyear=fiscalyear.rec_name))
 
             periods = Period.search([
                     ('fiscalyear', '=', fiscalyear.id),
@@ -300,12 +292,12 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
             Period.close(periods)
 
             with Transaction().set_context(fiscalyear=fiscalyear.id,
-                    date=None, cumulate=True):
+                    date=None, cumulate=True, journal=None):
                 accounts = Account.search([
                         ('company', '=', fiscalyear.company.id),
                         ])
-                deferrals += filter(None, (fiscalyear.get_deferral(a)
-                        for a in accounts))
+                deferrals += [_f for _f in (fiscalyear.get_deferral(a)
+                        for a in accounts) if _f]
         Deferral.save(deferrals)
 
     @classmethod
@@ -323,7 +315,9 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
                         ('state', '!=', 'open'),
                         ('company', '=', fiscalyear.company.id),
                         ]):
-                cls.raise_user_error('reopen_error')
+                raise FiscalYearReOpenError(
+                    gettext('account.msg_reopen_fiscalyear_later',
+                        fiscalyear=fiscalyear.rec_name))
 
             deferrals = Deferral.search([
                 ('fiscalyear', '=', fiscalyear.id),
@@ -342,16 +336,6 @@ class FiscalYear(Workflow, ModelSQL, ModelView):
         Period.lock(periods)
 
 
-class FiscalYearLine(ModelSQL):
-    'Fiscal Year - Move Line'
-    __name__ = 'account.fiscalyear-account.move.line'
-    _table = 'account_fiscalyear_line_rel'
-    fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
-            ondelete='CASCADE', select=True)
-    line = fields.Many2One('account.move.line', 'Line', ondelete='RESTRICT',
-            select=True, required=True)
-
-
 class BalanceNonDeferralStart(ModelView):
     'Balance Non-Deferral'
     __name__ = 'account.fiscalyear.balance_non_deferral.start'
@@ -362,7 +346,11 @@ class BalanceNonDeferralStart(ModelView):
     journal = fields.Many2One('account.journal', 'Journal', required=True,
         domain=[
             ('type', '=', 'situation'),
-            ])
+            ],
+        context={
+            'company': Eval('company', -1),
+            },
+        depends=['company'])
     period = fields.Many2One('account.period', 'Period', required=True,
         domain=[
             ('fiscalyear', '=', Eval('fiscalyear')),
@@ -372,7 +360,8 @@ class BalanceNonDeferralStart(ModelView):
     credit_account = fields.Many2One('account.account', 'Credit Account',
         required=True,
         domain=[
-            ('kind', '!=', 'view'),
+            ('type', '!=', None),
+            ('closed', '!=', True),
             ('company', '=', Eval('company', -1)),
             ('deferral', '=', True),
             ],
@@ -380,7 +369,8 @@ class BalanceNonDeferralStart(ModelView):
     debit_account = fields.Many2One('account.account', 'Debit Account',
         required=True,
         domain=[
-            ('kind', '!=', 'view'),
+            ('type', '!=', None),
+            ('closed', '!=', True),
             ('company', '=', Eval('company', -1)),
             ('deferral', '=', True),
             ],
@@ -445,7 +435,8 @@ class BalanceNonDeferral(Wizard):
             accounts = Account.search([
                     ('company', '=', self.start.fiscalyear.company.id),
                     ('deferral', '=', False),
-                    ('kind', '!=', 'view'),
+                    ('type', '!=', None),
+                    ('closed', '!=', True),
                     ])
         lines = []
         for account in accounts:
@@ -471,13 +462,70 @@ class BalanceNonDeferral(Wizard):
     def do_balance(self, action):
         self.create_move()
         action['pyson_domain'] = PYSONEncoder().encode([
-                ('origin', '=', str(self.start.fiscalyear)),
+                ('move.origin', '=', str(self.start.fiscalyear)),
                 ])
         return action, {}
 
 
+class CreatePeriodsStart(ModelView):
+    "Create Periods Start"
+    __name__ = 'account.fiscalyear.create_periods.start'
+    frequency = fields.Selection([
+            ('monthly', "Monthly"),
+            ('quarterly', "Quarterly"),
+            ('other', "Other"),
+            ], "Frequency", sort=False, required=True)
+    interval = fields.Integer("Interval", required=True,
+        states={
+            'invisible': Eval('frequency') != 'other',
+            },
+        depends=['frequency'],
+        help="The length of each period, in months.")
+    end_day = fields.Integer("End Day", required=True,
+        help="The day of the month on which periods end.\n"
+        "Months with fewer days will end on the last day.")
+
+    @classmethod
+    def default_frequency(cls):
+        return 'monthly'
+
+    @classmethod
+    def default_end_day(cls):
+        return 31
+
+    @classmethod
+    def frequency_intervals(cls):
+        return {
+            'monthly': 1,
+            'quarterly': 3,
+            'other': None,
+            }
+
+    @fields.depends('frequency', 'interval')
+    def on_change_frequency(self):
+        if self.frequency:
+            self.interval = self.frequency_intervals()[self.frequency]
+
+
+class CreatePeriods(Wizard):
+    "Create Periods"
+    __name__ = 'account.fiscalyear.create_periods'
+    start = StateView('account.fiscalyear.create_periods.start',
+        'account.fiscalyear_create_periods_start_view_form', [
+            Button("Cancel", 'end', 'tryton-cancel'),
+            Button("Create", 'create_periods', 'tryton-ok', default=True),
+            ])
+    create_periods = StateTransition()
+
+    def transition_create_periods(self):
+        self.model.create_period(
+            [self.record], self.start.interval, self.start.end_day)
+        return 'end'
+
+
 def month_delta(d1, d2):
-    return (d1.year - d2.year) * 12 + d1.month - d2.month
+    month_offset = 1 if d1.day < d2.day else 0
+    return (d1.year - d2.year) * 12 + d1.month - d2.month - month_offset
 
 
 class RenewFiscalYearStart(ModelView):
@@ -554,7 +602,12 @@ class RenewFiscalYear(Wizard):
             'periods': [],
             }
         previous_sequence = self.start.previous_fiscalyear.post_move_sequence
-        sequence, = Sequence.copy([previous_sequence])
+        sequence, = Sequence.copy([previous_sequence],
+            default={
+                'name': lambda data: data['name'].replace(
+                    self.start.previous_fiscalyear.name,
+                    self.start.name)
+                })
         if self.start.reset_sequences:
             sequence.number_next = 1
         else:
@@ -572,10 +625,12 @@ class RenewFiscalYear(Wizard):
         periods = [p for p in self.start.previous_fiscalyear.periods
             if p.type == 'standard']
         months = month_delta(fiscalyear.end_date, fiscalyear.start_date) + 1
-        if len(periods) == months:
-            FiscalYear.create_period([fiscalyear])
-        elif len(periods) == months / 3:
-            FiscalYear.create_period_3([fiscalyear])
+        interval = months / len(periods)
+        end_day = max(p.end_date.day
+            for p in self.start.previous_fiscalyear.periods
+            if p.type == 'standard')
+        if interval.is_integer():
+            FiscalYear.create_period([fiscalyear], interval, end_day)
         return fiscalyear
 
     def do_create_(self, action):
