@@ -290,8 +290,8 @@ class TaxCode(ActivePeriodMixin, tree(), ModelSQL, ModelView):
                 if child.template:
                     if not child.template_override:
                         if child.template.parent:
-                            parent = template2tax_code[
-                                child.template.parent.id]
+                            parent = template2tax_code.get(
+                                child.template.parent.id)
                         else:
                             parent = None
                         old_parent = (
@@ -1129,7 +1129,7 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
                 if child.template:
                     if not child.template_override:
                         if child.template.parent:
-                            parent = template2tax[child.template.parent.id]
+                            parent = template2tax.get(child.template.parent.id)
                         else:
                             parent = None
                         old_parent = (
